/* (C) 2003 - 2007 by Marcin Wiacek www.mwiacek.com */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>

#include "../cfg/config.h"
#include "../common/misc/files.h"
#include "../common/misc/misc.h"
#include "../common/misc/coding/coding.h"
#include "../common/gsmstate.h"
#include "../common/service/gsmmisc.h"
#include "../common/service/backup/gsmback.h"
#include "../common/service/smsmms/gsmsms.h"
#include "../common/phone/nokia/ndct34.h"

#include "gplus.h"
#include "gmsg.cpp"
#include "gfiles.cpp"
#include "gbackup.cpp"
#include "gcal.cpp"
#include "gpbk.cpp"

char *DayOfWeekStr(int Year, int Month, int Day)
{
	switch (DayOfWeek(Year,Month,Day)) {
	case 1: return "Monday";
	case 2: return "Tuesday";
	case 3: return "Wednesday";
	case 4: return "Thursday";
	case 5: return "Friday";
	case 6: return "Saturday";
	case 7: return "Sunday";
	default:return "";
	}
}

void PrintError(GSM_Error error)
{
	if (error.Code != GSM_ERR_NONE) {
		printf("%s\n",GSM_GetErrorInfo(error));
		exit(-1);
	}
}

void PrintStdErr(char *format, ...)
{
	va_list	argp;
	int 	chars,i;
	char 	buff[200];

	va_start(argp, format);
	chars = vsprintf(buff,format,argp);
	va_end(argp);

	fprintf(stderr,"%c%s",13,buff);
	if (PrevStdErr!=0) for(i=strlen(buff);i<PrevStdErr;i++) fprintf(stderr," ");
	fprintf(stderr,"%c",13);	
	PrevStdErr = chars;
}

void PrintTimeLeftReset()
{
	told	  = 0;
	toldfirst = TRUE;
	GSM_GetCurrentDateTime(&DTStart);
}

void PrintTimeLeft(char *buffer, int small_, int large_)
{
	time_t 		t;
	GSM_DateTime 	DTNow;
	char 		buff[200];

	sprintf(buff,"%s",buffer);
	sprintf(buff+strlen(buff),"%i %%",small_*100/large_);

	GSM_GetCurrentDateTime(&DTNow);

	t=GSMDateTime2TimeT(&DTNow)-GSMDateTime2TimeT(&DTStart);
	sprintf(buff+strlen(buff),", time %02i:",t/60);
	sprintf(buff+strlen(buff),"%02i",t%60);
	t=t*(large_-small_)/small_;
	told = t;
	if (told!=0) {
		sprintf(buff+strlen(buff),", %02i:",told/60);
		sprintf(buff+strlen(buff),"%02i left",told%60);
	}
	PrintStdErr("%s",buff);
}

#define DCT4GetUEM 50000

GSM_Error DCT4ReplyGetUEM(int MsgLength, unsigned char MsgType, unsigned char *MsgBuffer, void *Struct, int RequestID)
{
	// Struct is pointer to any structure given during frame sending
	char *Buf = (char *)Struct;

	//we must check Type of message
	//and how long it is
	if (MsgType == 0x1B && MsgLength > 4) {
		//message is enough long and now we check subtype
		if (MsgBuffer[3] == 0x08) {
			//if we didn't ask for it return error
			if (RequestID != DCT4GetUEM) return GSM_Return_Error(GSM_ERR_FRAME_NOT_REQUESTED);
			//we wanted it now and we fill our structure and return
			strcpy(Buf,(const char *)MsgBuffer+10);
			return GSM_Return_Error(GSM_ERR_NONE);
		}
	}

	//we must tell Gammu+, that we want to parse this frame by internal
	//module
	return GSM_Return_Error(GSM_ERR_FRAME_TYPE_UNKNOWN);
}

void Identify(GSM_StateMachine *s, GSM_GPlusConfig *CFG, int argc, char *argv[])
{
        GSM_Error 		error;
	unsigned char 		IMEI[20], Model[50], Model2[50], Firm[50], Man[50], Firm2[50];

#ifdef DEBUGxxxx
    GSM_SMSMMSDecodedEntry Decoded,Decodedx;
    GSM_SMSMMSDecodedSubEntry *Decoded2;
    GSM_SMSList List;
    int left;

    Decoded2 = new GSM_SMSMMSDecodedSubEntry;
    Decoded2->Type = MSG_MMSSMS_File;
    Decoded2->Text.append(StringToUnicodeReturn("text/plain"));
    UnicodeToUTF8(StringToUnicodeReturn("ala ma kota laka test sms"), &Decoded2->File.Buffer);
    Decoded.Add(Decoded2);

    Decoded2 = new GSM_SMSMMSDecodedSubEntry;
    Decoded2->Type = MSG_MMSSMS_File;
    Decoded2->Text.append(StringToUnicodeReturn("text/plain"));
    Decoded2->TextBold=TRUE;
    UnicodeToUTF8(StringToUnicodeReturn("test"), &Decoded2->File.Buffer);
    Decoded.Add(Decoded2);

//147
    Decoded.SaveToSMS(&List, SMS_EMS_Short,&left);

printf("%i left\n",left);

	Decodedx.ClearAll();
        error=Decodedx.ReadFromSMS(&List);

Decoded2=NULL;
while (Decodedx.GetNext(&Decoded2)) {
	printf("sms %s\n",Decoded2->File.Buffer.data());
}

Decoded2=NULL;
while (Decoded.GetNext(&Decoded2)) {
	printf("%i %i %i\n",
	Decoded2->SMSPackedNum,
	Decoded2->SMSPackedTextStartup,
	Decoded2->SMSPackedTextLength);
}


GSM_SMSListSubEntry *SMS2;
SMS2=NULL;
List.GetNext(&SMS2);
printf("%i\n",SMS2->GetSMS()->UserData.size());
for(int i=0;i<SMS2->GetSMS()->UserData.size();i++) printf("%02x %c ",SMS2->GetSMS()->UserData.data()[i],SMS2->GetSMS()->UserData.data()[i]);
#endif

	printf("Gammu+\n");
	printf("  Version      : %s",VERSION);
	printf(" built %s %s",__TIME__,__DATE__);
	if (strlen(GetCompiler()) != 0) printf(" using %s",GetCompiler());
	printf("\n");
	if (strlen(GetOS()) != 0) printf("  Run on       : %s\n",GetOS());
	
	error = s->Open(CFG);
	PrintError(error);

	if (!strcmp(s->Protocols->CurrentName,"irdaphonet")) {
		printf("  Connection   : infrared (%s)",s->Protocols->CurrentName);
#ifndef WIN32
		printf(" on %s",s->Devices->CurrentName);
#endif
	} else if (!strcmp(s->Protocols->CurrentName,"bluephonet")) {
		printf("  Connection   : bluetooth (%s)",s->Protocols->CurrentName);
#ifndef WIN32
		printf(" on %s",s->Devices->CurrentName);
#endif
	} else {
		printf("  Connection   : %s",s->Protocols->CurrentName);
	}

//	error = s->Phones->Current->Close();
//return;

	printf("\n  Phone module : %s\n",s->Phones->Current->ModuleName);

	printf("Phone\n");

	error = s->Phones->Current->GetIMEI(IMEI);
	PrintError(error);
	printf("  IMEI         : %s\n",IMEI);


	error = s->Phones->Current->GetManufacturer(Man);
	PrintError(error);
	error = s->Phones->Current->GetCodeNameModel(Model2);
	PrintError(error);
	error = s->Phones->Current->GetModel(Model);
	if (error.Code == GSM_ERR_NONE) {
//	PrintError(error);
		printf("  Model        : %s %s (%s)\n",Man, Model, Model2);
	} else {
		printf("  Model        : %s %s\n",Man, Model2);
	}

	error = s->Phones->Current->GetFirmwareVersion(Firm);
	PrintError(error);
	printf("  Firmware     : %s",Firm);
	error = s->Phones->Current->GetFirmwareDate(Firm2);
	if (error.Code == GSM_ERR_NONE) {
//	PrintError(error);
		printf(" (%s)\n",Firm2);
	}

	if (!strcmp(s->Phones->Current->ModuleName,"dct4 (n6510)")) {
		//we set handler
		s->SetUserReply(DCT4ReplyGetUEM);
		error = s->Phones->Current->Write((unsigned char *)"\x00\x03\x02\x07\x00\x08",6, 0x1B, 4, DCT4GetUEM, Model);
		PrintError(error);
		s->SetUserReply(NULL);		
		printf("  UEM          : %s\n",Model);
	}
}

void Monitor(GSM_StateMachine *s, GSM_GPlusConfig *CFG, int argc, char *argv[])
{
        GSM_Error 		error;
	GSM_PBKStatus 		Status;

	error = s->Open(CFG);
	PrintError(error);

	while (1) {
		Status.Memory=MEM_PHONE;
		error = s->Phones->Current->GetPBKStatus(&Status);
		if (error.Code != GSM_ERR_NOT_SUPPORTED) PrintError(error);

		Status.Memory=MEM_SIM;
		error = s->Phones->Current->GetPBKStatus(&Status);
		if (error.Code != GSM_ERR_NOT_SUPPORTED) PrintError(error);
	}
}

void GetDateTime(GSM_StateMachine *s, GSM_GPlusConfig *CFG, int argc, char *argv[])
{
        GSM_Error 		error;
	GSM_DateTime		DT;

	error = s->Open(CFG);
	PrintError(error);

	error = s->Phones->Current->GetDateTime(&DT);
	PrintError(error);

	printf("Date & time %02i:%02i:%02i %02i-%02i-%04i %s\n",
		DT.Hour,DT.Minute,DT.Second,DT.Day,DT.Month,DT.Year,
		DayOfWeekStr(DT.Year,DT.Month,DT.Day));
}

void Modules(GSM_StateMachine *s, GSM_GPlusConfig *CFG, int argc, char *argv[])
{
	list<GSM_Device_Info>::iterator 	devinfo;
	list<GSM_Protocol_Info>::iterator 	proinfo;
	list<GSM_Phone_Info>::iterator 		phoinfo;
        GSM_Device              		*dev;
        GSM_Protocol            		*pro;
        GSM_Phone               		*pho;

        printf("Available device modules\n");
        dev = NULL;
        while(1) {
                if (!s->Devices->GetNext(&dev)) break;
		for (devinfo=dev->Info.begin(); devinfo!=dev->Info.end(); ++devinfo) {
                        printf("  \"%s\"\n",(*devinfo).Device);
		}
        }

        printf("Available protocol modules\n");
        pro = NULL;
        while(1) {
                if (!s->Protocols->GetNext(&pro)) break;
		for (proinfo=pro->Info.begin(); proinfo!=pro->Info.end(); ++proinfo) {
                        printf("  \"%s\" dev \"%s\"\n",(*proinfo).Protocol,(*proinfo).Device);
	        }
        }

        printf("Available phone modules\n");
        pho = NULL;
        while(1) {
                if (!s->Phones->GetNext(&pho)) break;
	        printf("%s\n",pho->ModuleName);
		for (phoinfo=pho->Info.begin(); phoinfo!=pho->Info.end(); ++phoinfo) {
                        printf("  \"%s\" prot \"%s\" cname \"%s\" devname \"%s\" feat \"%s\"\n",
                                (*phoinfo).Model,(*phoinfo).Protocol, (*phoinfo).CodeNameModel, 
                                (*phoinfo).DeviceModel, (*phoinfo).Features);
	        }
        }
}

void FindNetworkNumber(GSM_StateMachine *s, GSM_GPlusConfig *CFG, int argc, char *argv[])
{
unsignedstring Number;
unsignedstring Network;

FindNumberNetwork(&Number, &Network);
}

void Help(GSM_StateMachine *s, GSM_GPlusConfig *CFG, int argc, char *argv[]);

static CommandLineFunction CommandLineFunctions[] = {
	"--identify",		0, 0, Identify,      "",
	"--modules",		0, 0, Modules,       "",
	"--help",		0, 0, Help,   	     "",
	"--monitor",		0, 0, Monitor, 	     "",
#ifdef DEBUG
	"--testsms",		1, 1, TestSMS,       "file",
#endif
	"--getmemory",		2, 3, GetMemory,     "ME|SM location",
	"--restore",		1, 1, Restore,       "",
	"--backup",		1, 1, Backup,        "",
	"--getallcalendar",	0, 0, GetAllCalendar,"",
	"--getallnotes",	0, 0, GetAllNotes,   "",
	"--getdatetime",	0, 0, GetDateTime,   "",
	"--clearall",		0, 0, ClearAll,	     "",
	"--getmmsfolders",	0, 0, GetMMSFolders, "",
	"--readmmsfile",	1, 2, ReadMMSFile,   "file [-save]",
	"--getsmsstatus",	0, 0, GetSMSStatus,  "",
	"--getallsms",		0, 0, GetAllSMS,     "",
	"--geteachsms",		0, 0, GetEachSMS,    "",
	"--getsmsfolders",	0, 0, GetSMSFolders, "",
	"--getsmsc",		1, 1, GetSMSC, 	     "",
	"--addsms",		0, 0, AddSMS, 	     "",
	"--sendsms",		2, 2, SendSMS, 	     "number text",
	"--deletesms",		2, 2, DeleteSMS,     "ID",
	"--getfilesystem",	0, 0, GetFileSystem, "",
	"--getfile",		1, 1, GetFile,       "fileID",
	"--addfile",		2, 2, AddFile,       "ParentFolderID FileName",
	"--addfolder",		2, 2, AddFolder,     "ParentFolderID Name",
	"--deletefolder",	1, 1, DeleteFolder,  "ID",
	"--deletefile",		1, 1, DeleteFile,    "ID",
	"--displayjava",	0, 0, DisplayJava,   "",
	"--addjavagame",	1, 1, AddJavaGame,   "name(without JAD/JAR extensions)",
	"--addjavaapp",		1, 1, AddJavaApp,    "name(without JAD/JAR extensions)",
//	"--findnetworknumber",	1, 1, FindNetworkNumber,    "number",
#ifdef POSTGRESQL
	"--smsmmsd",		4, 5, SMSMMSD,       "ip user password db [port]",
#endif

	"",			0, 0, NULL,	     "",
};

void Help(GSM_StateMachine *s, GSM_GPlusConfig *CFG, int argc, char *argv[])
{
	int i = 0;

	while (CommandLineFunctions[i].Parameter[0] != 0) {
		printf("%s %s\n",CommandLineFunctions[i].Parameter,CommandLineFunctions[i].Params);
		i++;
	}
}

int main2(int argc, char *argv[])
{
        GSM_StateMachine 	s(DebFile,"",UseDeb);
	GSM_GPlusConfig 		CFGFile;
        int 			i=0,startarg=0;
	unsignedstring		Stable,Test;

	Stable.append((unsigned char *)"[CMD LINE  :",12);
	for (i=1;i<argc;i++) {
		Stable.push_back(' ');
		Stable.push_back('\"');
		Stable.append((unsigned char *)argv[i],strlen(argv[i]));
		Stable.push_back('\"');
	}
	Stable.append((unsigned char *)"]\n",2);
	s.Debug->Deb((char *)Stable.data());
	Stable.clear();

	i=0;

	if (argc > 1 && StringCaseCmp(argv[1],"nothing",-1)) {
		startarg++;
	} else if (argc > 1 && StringCaseCmp(argv[1],"textall",-1)) {
		startarg++;
	}
	s.ReadCfg(1,&CFGFile);

	if (CFGFile.RSSLevel[0]!=0) {
		CheckRSS(&Stable, &Test);
		if (StringCaseCmp(CFGFile.RSSLevel, "teststable", -1)) {
			if (Test.length()!=0) printf("You are using Gammu+ version %s, but there is already newer test version %s. See www.gammu.org for more.\n",VERSION,Test.data());
		}
		if (StringCaseCmp(CFGFile.RSSLevel, "teststable", -1) ||
			StringCaseCmp(CFGFile.RSSLevel, "stable", -1)) {
			if (Stable.length()!=0) printf("You are using Gammu+ version %s, but there is already newer stable version %s. See www.gammu.org for more.\n",VERSION,Stable.data());
		}
	}

	if (argc-startarg==1) {
		Help(&s,&CFGFile,argc-2-startarg,argv+2+startarg);
		return 0;
	}

	while(1) {
		if (CommandLineFunctions[i].Parameter[0] == 0) break;
		if (!StringCaseCmp(argv[1+startarg],CommandLineFunctions[i].Parameter,-1)) {
			i++;
			continue;
		}
		if (argc-2-startarg < CommandLineFunctions[i].MinArguments) {
			printf("Give more arguments (required ");
			if (CommandLineFunctions[i].MinArguments!=CommandLineFunctions[i].MaxArguments) {
				printf("%i to %i",
					CommandLineFunctions[i].MinArguments,
					CommandLineFunctions[i].MaxArguments);
			} else {
				printf("%i",CommandLineFunctions[i].MaxArguments);
			}
			if (CommandLineFunctions[i].Params[0]!=0) {
				printf(": %s",CommandLineFunctions[i].Params);
			}
			printf(")\n");
			return 0;
		}
		if (argc-2-startarg > CommandLineFunctions[i].MaxArguments) {
			printf("Too many arguments (required ");
			if (CommandLineFunctions[i].MinArguments!=CommandLineFunctions[i].MaxArguments) {
				printf("%i to %i",
					CommandLineFunctions[i].MinArguments,
					CommandLineFunctions[i].MaxArguments);
			} else {
				printf("%i",CommandLineFunctions[i].MaxArguments);
			}
			if (CommandLineFunctions[i].Params[0]!=0) {
				printf(": %s",CommandLineFunctions[i].Params);
			}
			printf(")\n");
			return 0;
		}
		CommandLineFunctions[i].Function(&s,&CFGFile,argc-2-startarg,argv+2+startarg);
		return 0;
	}

	Help(&s,&CFGFile,argc-2-startarg,argv+2+startarg);
	return 0;
}

int main(int argc, char *argv[])
{
#ifdef DEBUG
	UseDeb 	= true;
	DebFile = stdout;
#endif
	if (argc > 1 && StringCaseCmp(argv[1],"nothing",-1)) {
		UseDeb 	= false;
		DebFile = NULL;
	} else if (argc > 1 && StringCaseCmp(argv[1],"textall",-1)) {
		UseDeb 	= true;
		DebFile = stdout;
	}

	main2(argc,argv);
}
